@@ -1,6 +1,6 @@
 /*************************************************************************
  *
- * Copyright (c) 2012-2014 Kohei Yoshida
+ * Copyright (c) 2012-2016 Kohei Yoshida
  *
  * Permission is hereby granted, free of charge, to any person
  * obtaining a copy of this software and associated documentation
@@ -29,17 +29,17 @@
 #define MDDS_MULTI_TYPE_VECTOR_TYPES_HPP
 
 #include "default_deleter.hpp"
-#include "compat/unique_ptr.hpp"
 #include "global.hpp"
 
 #include <algorithm>
+#include <cassert>
+#include <memory>
 
 #ifdef MDDS_MULTI_TYPE_VECTOR_USE_DEQUE
 #include <deque>
 #else
 #include <vector>
 #endif
-#include <boost/noncopyable.hpp>
 
 #if defined(MDDS_UNIT_TEST) || defined (MDDS_MULTI_TYPE_VECTOR_DEBUG)
 #include <iostream>
@@ -151,6 +151,11 @@ public:
         return get(block).m_array.at(pos);
     }
 
+    static typename store_type::size_type size(const base_element_block& block)
+    {
+        return get(block).m_array.size();
+    }
+
     static iterator begin(base_element_block& block)
     {
         return get(block).m_array.begin();
@@ -227,6 +232,11 @@ public:
         val = get(blk).m_array[pos];
     }
 
+    static value_type get_value(const base_element_block& blk, size_t pos)
+    {
+        return get(blk).m_array[pos];
+    }
+
     static void append_value(base_element_block& blk, const _Data& val)
     {
         get(blk).m_array.push_back(val);
@@ -250,7 +260,13 @@ public:
 
     static void resize_block(base_element_block& blk, size_t new_size)
     {
-        static_cast<_Self&>(blk).m_array.resize(new_size);
+        store_type& st = get(blk).m_array;
+        st.resize(new_size);
+
+        // Test if the vector's capacity is larger than twice its current
+        // size, and if so, shrink its capacity to free up some memory.
+        if (new_size < (st.capacity() / 2))
+            st.shrink_to_fit();
     }
 
 #ifdef MDDS_UNIT_TEST
@@ -393,8 +409,7 @@ public:
     static void shrink_to_fit(base_element_block& block)
     {
 #ifndef MDDS_MULTI_TYPE_VECTOR_USE_DEQUE
-        store_type& blk = get(block).m_array;
-        store_type(blk).swap(blk);
+        get(block).m_array.shrink_to_fit();
 #endif
     }
 
@@ -428,12 +443,13 @@ public:
 
     static _Self* clone_block(const base_element_block& blk)
     {
+        // Use copy constructor to copy the data.
         return new _Self(get(blk));
     }
 };
 
 template<typename _Self, element_t _TypeId, typename _Data>
-class noncopyable_element_block : public element_block<_Self, _TypeId, _Data>, private boost::noncopyable
+class noncopyable_element_block : public element_block<_Self, _TypeId, _Data>
 {
     typedef element_block<_Self,_TypeId,_Data> base_type;
 protected:
@@ -445,6 +461,9 @@ protected:
     noncopyable_element_block(const _Iter& it_begin, const _Iter& it_end) : base_type(it_begin, it_end) {}
 
 public:
+    noncopyable_element_block(const noncopyable_element_block&) = delete;
+    noncopyable_element_block& operator=(const noncopyable_element_block&) = delete;
+
     static _Self* clone_block(const base_element_block&)
     {
         throw element_block_error("attempted to clone a noncopyable element block.");
@@ -537,7 +556,7 @@ struct managed_element_block : public copyable_element_block<managed_element_blo
         if (init_size > 1)
             throw general_error("You can't create a managed block with initial value.");
 
-        unique_ptr<self_type> blk(new self_type(init_size));
+        std::unique_ptr<self_type> blk = make_unique<self_type>(init_size);
         if (init_size == 1)
             set_value(*blk, 0, val);
 
@@ -586,7 +605,7 @@ struct noncopyable_managed_element_block : public noncopyable_element_block<nonc
         if (init_size > 1)
             throw general_error("You can't create a managed block with initial value.");
 
-        unique_ptr<self_type> blk(new self_type(init_size));
+        std::unique_ptr<self_type> blk = make_unique<self_type>(init_size);
         if (init_size == 1)
             set_value(*blk, 0, val);
 
